/*

-------------------------------------------
Makro: DS_VARS
Nzev: Testuje promnn v datasetu
       Tests existence of dataset variables
Verze: 2.2
Autor: jan.ruzicka@cze.sas.com
-------------------------------------------

-------------------
POPIS - DESCRIPTION
-------------------

Tato makro funkce testuje existenci promnnch v datasetu.
This macro tests existence of dataset variables.

----------------
SYNTAXE - SYNTAX
----------------

%ds_vars
(
    data,      *** specifikace datasetu
                   dataset specification
    varlist,   *** seznam promnnch k testovn
                   variable list to test
    typelist=, *** seznam typ promnnch k testovn
                   type list of the variables
    debug=yes  *** vypisovn do logu
                   list errors to log
)

----------------------------------
HISTORIE ZMN - HISTORY OF CHANGES
----------------------------------

01.11.2003  Verze 1.40
            + zapracovn do jednotnho frameworku SAS maker
              rewriten as part of the framework
15.01.2004  Verze 1.41
            + popis v anglitin
              description in english
31.10.2008  Verze 2.2
            - zmna hlen do logu z typu ERROR: na INFO:
              changed log message type from ERROR: to INFO:

---------------------------------
PLNY NA VYLEPEN - FUTURE PLANS
---------------------------------

-------------------
PKLADY - EXAMPLES
-------------------

*** 1 - Inside macro definition;

%macro test;
%if %ds_vars(sashelp.air,date air) %then %put In SASHELP.AIR exists variables DATE a AIR;
%mend test;
%test

*** 2 - Variables exists, but variable NAME is shorter then 10 and isn't character;

%put %ds_vars(sashelp.class,sex name weight,typelist=$ $10 $);

----------------
POZNMKY - NOTES
----------------

Pipomnky a npady k vylepen vtny.
Ideas to improve are welcome.

*/

%macro ds_vars
(
    data,
    varlist,
    typelist=,
    debug=yes
) / des='Tests existence of dataset variables';

%local result dsid rc i varlen;

%let result=1;

%let debug=%qupcase(&debug);

%let dsid=%sysfunc(open(&data));
%if &dsid %then %do;
    %let i=1;
    %do %while(%qscan(&varlist,&i,%str( )) ne);
        %let rc=%sysfunc(varnum(&dsid,%qscan(&varlist,&i,%str( ))));
        %if &rc=0 %then %do;
            %let result=0;
            %if &debug=YES %then %put INFO: Variable "%qscan(&varlist,&i,%str( ))" does not exists in dataset %upcase(&data).;
        %end;
        %else %if %qscan(&typelist,&i,%str( )) ne %then %do;
            %let varlen=%sysfunc(varlen(&dsid,&rc));
            %if %sysfunc(vartype(&dsid,&rc))=C %then %do;
                %if %qsubstr(%qscan(&typelist,&i,%str( )),1,1) ne $ %then %do;
                    %let result=0;
                    %if &debug=YES %then %put INFO: Character variable "%qscan(&varlist,&i,%str( ))" in dataset &data does not match required type (numeric?).;
                %end;
                %else %if %length(%qscan(&typelist,&i,%str( )))>1 %then %if %qsubstr(%qscan(&typelist,&i,%str( )),2)>&varlen %then %do;
                    %let result=0;
                    %if &debug=YES %then %put INFO: Character variable "%qscan(&varlist,&i,%str( ))" in dataset &data does not match required length (%qsubstr(%qscan(&typelist,&i,%str( )),2)).;
                %end;
            %end;
            %else %do;
                %if %qsubstr(%qscan(&typelist,&i,%str( )),1,1) ne $ %then %do;
                    %if %qscan(&typelist,&i,%str( ))>&varlen %then %do;
                        %let result=0;
                        %if &debug=YES %then %put INFO: Numeric variable "%qscan(&varlist,&i,%str( ))" in dataset &data does not match required length (%qscan(&typelist,&i,%str( ))).;
                    %end;
                %end;
                %else %do;
                    %let result=0;
                    %if &debug=YES %then %put INFO: Numeric variable "%qscan(&varlist,&i,%str( ))" in dataset &data does not match required type (character?).;
                %end;
            %end;
        %end;
        %let i=%eval(&i+1);
    %end;
    %let dsid=%sysfunc(close(&dsid));
%end;
%else %do;
    %let result=0;
    %if &debug=YES %then %put ERROR: Cannot read dataset %upcase(&data).;
%end;

%mexit:

&result

%mend ds_vars;
