/*

--------------------------------------------------
Makro: MODEL_PROFILEVAR
Nzev: Popis vztahu promnn k modelovan promnn
       Describes relation variable to target
Verze: 2.21
Autor: jan.ruzicka@cze.sas.com
--------------------------------------------------

-------------------
POPIS - DESCRIPTION
-------------------

Makro vytvo popis vztahu prediktoru na clov vysvtlovan promnn
Vsledn reporty se mohou odkazovat na reporty o DW (viz. %RUN_DESCRIBE,
%RUN_DESCRIBEVAR, %RUN_FORMATS).

Macro creates reports relation variable to target variable. Reports could
reference other DW reports (look %RUN_RESCRIBE, %RUN_DESCRIBEVAR, %RUN_FORMATS).


Pedpokldan struktura adres:
Assumed directory structure:

&rep_root\catalogs  - vsledn reporty o katalozch a polokch katalog
                      reports about catalogs and entries in DW
&rep_root\formats   - vsledn reporty o vytvench formtech a informtech
                      reports about formats and informats in DW
&rep_root\columns   - vsledn reporty o datech v tabulkch DW (podadrese dle knihoven)
                      reports about columns in DW

&rep_root\..\all_reports.css  - defaultn HTML stylesheet
                                default HTML stylesheet

----------------
SYNTAXE - SYNTAX
----------------

%model_profilevar
(
    data=,                   *** dataset(y) pro popis rozloen promnnch
                                 dataset(s) for analysis
    target=,                 *** clov promnn
                                 target variable name
    target_type=binary,      *** typ clov promnn (nyn pouze BINARY)
                                 target variable type (only BINARY now)
    target_0=0,              *** rove 0 clov promnn
                                 target variable 0 level value
    target_0_label=Good,     *** popisek rovn 0 clov promnn
                                 target variable 0 level label
    target_1=1,              *** rove 1 clov promnn
                                 target variable 1 level value
    target_1_label=Bad,      *** popisek rovn 1 clov promnn
                                 target variable 1 level label
    binary=,                 *** binrn vstupn promnn
                                 binary inputs
    ordinal=,                *** ordinln vstupn promnn
                                 ordinal inputs
    nominal=,                *** nominln vstupn promnn
                                 nominal inputs
    interval=,               *** spojit vstupn promnn
                                 interval inputs
    groups=10,               *** poet interval pro kategorizaci spojitch vstupnch promnnch
                                 number of ranks for interval input variables
    rep_out=,                *** adres pro reporty
                                 output directory
    rep_name=,               *** jmno kmenov HTML strnky (bez ppony .html)
                                 name of root HTML page (without .html extension)
    rep_root=,               *** zkladn adres report (dokumentace DW)
                                 report root directory (documentation of the DW)
    rep_root_url=,           *** URL zkladnho adrese report (dokumentace DW)
                                 report root directory URL (documentation of the DW)
    rep_css=all_reports.css, *** specifikace pouvanho kaskdnho stylu
                                 stylesheet specification
    dirsep=                  *** definice oddlovae adres (u Windows se pedpokld \)
                                 directory seporator specification (\ is supposed for Windows)
)

----------------------------------
HISTORIE ZMN - HISTORY OF CHANGES
----------------------------------

30.03.2003  Verze 1.00
30.01.2004  Verze 2.20
            + popis v anglitin
              description in english
            + zapracovn do jednotnho frameworku SAS maker
              rewriten as part of the framework
            - zmna nzvu makra
              change macro name
12.11.2007  Verze 2.21
            - oprava generovn graf pro SAS 9.1.3 SP4
              bugfix of charts generation for SAS 9.1.3 SP4
            - podpora UNIX
              UNIX support

---------------------------------
PLNY NA VYLEPEN - FUTURE PLANS
---------------------------------

-------------------
PKLADY - EXAMPLES
-------------------

*** Example 1;

data class;
set sashelp.class;
bad=(weight*.5)/(height*.0254)**2>=20;
attrib
    name   label='Name'
    sex    label='Sex'
    age    label='Age'
    height label='Height'
    weight label='Weight'
    bad    label="Bad / Good";
run;
options mprint;
%model_profilevar
(
    data=class,
    interval=height weight,
    binary=sex,
    ordinal=age,
    groups=3,
    target=bad,
    rep_out=c:\temp
)

----------------
POZNMKY - NOTES
----------------

Pro rozdlen spojit promnn do interval se vyuv makro %MODEL_RANK.
Pro zjitn informac o promnn (datasetu) se vyuvaj makra %VAR_LABEL, %VAR_FORMAT,
%DS_LIBNAME, %DS_MEMNAME.
Following macros are utilized: %MODEL_RANK, %VAR_LABEL, %VAR_FORMAT,
%DS_LIBNAME, %DS_MEMNAME.

Makro voliteln vyuv HTML vstupy z maker %RUN_DESCRIBE, %RUN_DESCRIBEVAR, %RUN_FORMATS.
Macro optionally references reports from %RUN_DESCRIBE, %RUN_DESCRIBEVAR, %RUN_FORMATS macros.

Pipomnky a npady k vylepen vtny.
Ideas to improve are welcome.

*/

%macro model_profilevar
(
    data=,
    target=,
    target_type=binary,
    target_0=0,
    target_0_label=Good,
    target_1=1,
    target_1_label=Bad,
    binary=,
    ordinal=,
    nominal=,
    interval=,
    groups=10,
    rep_out=,
    rep_name=,
    rep_root=,
    rep_root_url=,
    rep_css=all_reports.css,
    rep_all=,
    dirsep=
) / des='Describes relation variable to target';

%local i br input input_type order note rep_name fmtcat sp4;

%put;
%put NOTE: Macro MODEL_PROFILEVAR 2.21;
%put;

%let sp4=0;
%if "&sysvlong4"="9.01.01M3P02022006" %then %let sp4=1;
%if &sysver>=9.1 %then %if "%sysfunc(hotfix())"="SAS 9.1.3 Service Pack 4" %then %let sp4=1;


%* Testy parametr;

%if "&dirsep"="" and &sysscp=WIN %then %let dirsep=\;
%if "&dirsep"="" %then %do;
    %put ERROR: Directory separator specification is required (option DIRSEP=).;
    %goto mexit;
%end;

%if %sysfunc(fileexist(&rep_out&dirsep...&dirsep&rep_css))=0 %then
    %put WARNING: Style sheet &rep_out&dirsep...&dirsep&rep_css not exist.;

%let target_type=%qupcase(&target_type);
%if &target_type ne BINARY %then %do;
    %put ERROR: Only valid value for TARGET_TYPE= option is BINARY.
    %goto mexit;
%end;

%if "&rep_out"="" %then %let rep_out=.;
%if "&rep_name"="" %then %let rep_name=%sysfunc(lowcase(%ds_libname(&data).%ds_memname(&data).&target));
%if "&rep_root_url"="" %then %let rep_root_url=&rep_root;

/* Uprava kodovani target promenne */

data __temp / view=__temp;
attrib &target length=3 label="%var_label(&data,&target)";
set &data (rename=(&target=__&target));
%if %substr(%var_format(&data,&target),1,1)=$ %then %do;
    if __&target="&target_0" then &target=0;
    else if __&target="&target_1" then &target=1;
%end;
%else %do;
    if __&target=&target_0 then &target=0;
    else if __&target=&target_1 then &target=1;
%end;
else &target=.;
drop __&target;
run;

proc sql;
create table __profilevar
(
    order  num       label='Order',
    type   char(256) label='Type',
    name   char(32)  label='Name',
    iv     num       label='Information value',
    note   char(256) label='Note'
);
quit;

%do i=1 %to %eval(%util_countwords(&binary)+%util_countwords(&ordinal)+%util_countwords(&nominal)+%util_countwords(&interval));

    %if &i<=%util_countwords(&binary) %then %do;
        %let input=%scan(&binary,&i);
        %let input_type=BINARY;
        %let order=1;
    %end;
    %else %if &i<=%eval(%util_countwords(&binary)+%util_countwords(&ordinal)) %then %do;
        %let input=%scan(&ordinal,%eval(&i-%util_countwords(&binary)));
        %let input_type=ORDINAL;
        %let order=2;
    %end;
    %else %if &i<=%eval(%util_countwords(&binary)+%util_countwords(&ordinal)+%util_countwords(&nominal)) %then %do;
        %let input=%scan(&nominal,%eval(&i-%util_countwords(&binary)-%util_countwords(&ordinal)));
        %let input_type=NOMINAL;
        %let order=3;
    %end;
    %else %do;
        %let input=%scan(&interval,%eval(&i-%util_countwords(&binary)-%util_countwords(&ordinal)-%util_countwords(&nominal)));
        %let input_type=INTERVAL;
        %let order=4;
    %end;
    
    %if &input_type=INTERVAL %then %do;
        %model_rank(
            data=__temp,
            var=&input,
            groups=&groups,
            out=__bins,
            outvar=__bin,
            fmtout=work,
            fmtname=__bin,
            fmtvalue=interval
        );
        proc summary data=__bins;
        class __bin;
        output out=__bins (drop=_type_ rename=(_freq_=__n))
               median(&input)=&input
               sum(&target)=__n1;
        format &input __bin.;
        run;
    %end;
    %else %do;
        proc summary data=__temp;
        class &input;
        output out=__bins (drop=_type_ rename=(_freq_=__n))
               sum(&target)=__n1;
        run;
        data __bins;
        set __bins;
        __bin=&input;
        label __bin="Rank for variable &input";
        run;
    %end;

    data __bins;
    set __bins end=__finish;
    if _n_=1 then do;
        __all_n=__n;
        __all_n1=__n1;
        __all_n0=__n-__n1;
        retain __all_n:;
    end;
    else do;
        __p=__n/__all_n;
        __n0=__n-__n1;
        __p1=__n1/__all_n1;
        __p0=__n0/__all_n0;
        __r1=__n1/__n;
        __r0=__n0/__n;
        __woe=log((__p0)/(__p1))*100;
        __all_iv+(__p0-__p1)*__woe/100;
        output;
    end;
    if __finish then do;
        call symput('ggroups',compress(put(_n_-1,best.)));
        call symput('iv',compress(put(__all_iv,8.4)));
        call symput('br',compress(put(__all_n1/__all_n,best.)));
    end;
    attrib
        __n   label='N'
        __p   label='%'                     format=percent8.1
        __n1  label="N of &target_1_label"
        __n0  label="N of &target_0_label"
        __p1  label="% of &target_1_label"  format=percent8.1
        __p0  label="% of &target_0_label"  format=percent8.1
        __r1  label="&target_1_label rate"  format=percent8.1
        __r0  label="&target_0_label rate"  format=percent8.1
        __woe label='WOE'             format=8.2
        __bin label="Group of &input"
    ;
    drop __all:;
    run;

    data __chart (keep=__bin &input __sub __n __p __r);
    set __bins (keep=__bin &input __n0 __p0 __r0 __n1 __p1 __r1);
    length __sub $%sysfunc(max(%length(&target_0_label),%length(&target_1_label)));
    __sub="&target_0_label";
    __n=__n0;
    __p=__p0;
    __r=__r0;
    output;
    __sub="&target_1_label";
    __n=__n1;
    __p=__p1;
    __r=__r1;
    output;
    attrib
        __n   label='N'      format=8.0
        __p   label='%'      format=percent8.1
        __r   label='Rate'   format=percent8.1
        __sub label='Target'
    ;
    run;

    proc datasets nolist;
    delete gseg / memtype=catalog;
    quit;

    ods listing close;
    goptions reset=all ftext='arial' htext=1.5 ftitle='arial' htitle=2;

    proc gchart data=__chart;
    axis1 style=0;
    axis2 minor=none order=(0 to 1 by .25) label=none;
    axis3 minor=none label=none;
    axis4 minor=(n=4) label=none;
    where __sub="&target_1_label";
    hbar __bin / discrete sumvar=__r noframe nostats
                 maxis=axis1 raxis=axis3 autoref cref=graya0 clipref
                 name="__1";
    title "&target_1_label rates";
    run;
    where;
    hbar __bin / discrete subgroup=__sub sumvar=__n noframe nostats
                 maxis=axis1 raxis=axis3 autoref cref=graya0 clipref
                 name="__2";
    title "&target_1_label / &target_0_label frequencies";
    run;
    quit;
    proc gchart data=__bins;
    hbar __bin / discrete sumvar=__woe noframe nostats
                 maxis=axis1 raxis=axis4 autoref cref=graya0 clipref
                 name="__3";
    title "Weight of evidence";
    run;
    hbar __bin / discrete sumvar=__p1 noframe nostats
                 maxis=axis1 raxis=axis4 autoref cref=graya0 clipref
                 name="__4";
    title "&target_1_label distribution";
    run;
    quit;

    ods listing;

    %if "&dirsep"="\" %then %do;
        proc printto print='nul';
        run;
    %end;
    %else %if "&dirsep"="/" %then %do;
        proc printto print='/dev/null';
        run;
    %end;

    filename __out "&rep_out&dirsep&rep_name%sysfunc(lowcase(&input)).gif";
    goptions dev=gif gsfname=__out gsfmode=replace;

    %if &sp4 %then %do;
        ods html file="&rep_out&dirsep&rep_name..%sysfunc(lowcase(&input))1.html" (title="%upcase(&input) variable profile")
                 stylesheet=(url="../&rep_css") nogfootnote;
    %end;
    %else %do;
        ods html file="&rep_out&dirsep&rep_name..%sysfunc(lowcase(&input)).html" (title="%upcase(&input) variable profile")
                 stylesheet=(url="../&rep_css") nogfootnote;
    %end;

    proc report data=__bins nofs style(summary)=[htmlclass="Header"];
    columns ("Attributes of &input" __bin &input) ('Total' __n __p)
            ("&target_0_label" __n0 __p0) ("&target_1_label" __n1 __p1) ('Measures' __r1 __woe);
    define __bin / group;
    define &input / group;
    compute after;
    __r1.sum=&br;
    __woe.sum=.;
    endcomp;
    rbreak after / summarize;
    %if "%var_label(&data,&input)"="" %then title "&target_1_label / &target_0_label by %upcase(&input)";
    %else title "&target_1_label / &target_0_label by %var_label(&data,&input)";;
    %if %sysevalf(&iv<0.02) %then %let note=Unpredictive (IV = &iv, &ggroups groups);
    %else %if %sysevalf(&iv<0.1) %then %let note=Weak predictivity (IV = &iv, &ggroups groups);
    %else %if %sysevalf(&iv<0.3) %then %let note=Medium predictivity (IV = &iv, &ggroups groups);
    %else %if %sysevalf(&iv<0.5) %then %let note=Strong predictivity (IV = &iv, &ggroups groups);
    %else %let note=Overpredictive (IV = &iv, &ggroups groups);
    footnote "&note";
    run;

    %if %sysfunc(fileexist(&rep_root&dirsep.columns&dirsep%sysfunc(lowcase(%ds_libname(&data)&dirsep%ds_memname(&data).&input)).html)) %then %do;
        proc sql;
        insert into __profilevar values
            (&order,"<a href=""&rep_root_url/columns/%sysfunc(lowcase(%ds_libname(&data)/%ds_memname(&data).&input)).html"">&input_type</a>","&input",&iv,"&note");
        quit;
    %end;
    %else %do;
        proc sql;
        insert into __profilevar values (&order,"&input_type","&input",&iv,"&note");
        quit;
    %end;

    goptions device=gif;
    proc greplay nofs;
    footnote link="&rep_name..html" "All variables";
    igout gseg;
    tc sashelp.templt;
    template l2r2;
    treplay 1:__1 2:__2 3:__3 4:__4 name="";
    run;
    quit;
    title;
    footnote;

    ods html close;

    proc printto print=print;
    run;

    filename __out clear;

    %if &sp4 %then %do;
        filename __out "&rep_out&dirsep&rep_name..%sysfunc(lowcase(&input))1.html" lrecl=4096;

        data _null_;
        infile __out ;
        file "&rep_out&dirsep&rep_name..%sysfunc(lowcase(&input)).html" lrecl=4096;
        input;
        a=index(lowcase(_infile_),' src="template.gif" ');
        if a then _infile_=tranwrd(_infile_,' src="template.gif" '," src=""&rep_name%sysfunc(lowcase(&input)).gif"" ");
        put _infile_;
        run;

        data _null_;
        a=fdelete('__out');
        run;
        filename __out clear;
    %end;

%end;

proc sql undo_policy=none;
create table __profilevar as
select
    order,
    type,
    case label ne '' when 1
        then "<a href=""&rep_name.."||trim(lowcase(a.name))||'.html">'||trim(upcase(a.name))||' - '||trim(label)||'</a>'
        else "<a href=""&rep_name.."||trim(lowcase(a.name))||'.html">'||trim(upcase(a.name))||'</a>' end
    as name label='Label',
    iv,
    note,
    case format ne ''
        when 1 then case fmttype
            when 'char' then case

            %let i=1;
            %do %while (%scan(%sysfunc(getoption(fmtsearch)),&i,%str(( ))) ne);
                %let fmtcat=%scan(%sysfunc(getoption(fmtsearch)),&i,%str(( )));
when fileexist("&rep_root&dirsep.catalogs&dirsep%sysfunc(lowcase(&fmtcat))."||lowcase(substr(left(
translate(format,' ','$')),1,length(left(translate(format,' ','$.0123456789')))))||".formatc.html")
then "<a href=""&rep_root_url/catalogs/%sysfunc(lowcase(&fmtcat))."||lowcase(substr(left(translate(format,' ','$'))
,1,length(left(translate(format,' ','$.0123456789')))))||'.formatc.html">'||trim(format)||'</a>'
                %let i=%eval(&i+1);
            %end;

                when fileexist("&rep_root\formats\"||lowcase(substr(left(
                    translate(format,' ','$')),1,length(left(translate(format,' ','$.0123456789')))))||
                    ".formatc.html")
                then "<a href=""&rep_root\formats\"||lowcase(substr(left(translate(format,' ','$'))
                    ,1,length(left(translate(format,' ','$.0123456789')))))||'.formatc.html">'||
                    trim(format)||'</a>'
                else format end
            else case

            %let i=1;
            %do %while (%scan(%sysfunc(getoption(fmtsearch)),&i,%str(( ))) ne);
                %let fmtcat=%scan(%sysfunc(getoption(fmtsearch)),&i,%str(( )));
when fileexist("&rep_root&dirsep.catalogs&dirsep%sysfunc(lowcase(&fmtcat))."||lowcase(substr(format,1,
length(trimn(translate(format,' ','.0123456789')))))||".format.html")
then "<a href=""&rep_root_url/catalogs/%sysfunc(lowcase(&fmtcat))."||lowcase(substr(format,1,length(trimn(
translate(format,' ','.0123456789')))))||'.format.html">'||trim(format)||'</a>'
                %let i=%eval(&i+1);
            %end;

                when fileexist("&rep_root\formats\"||lowcase(substr(format,1,
                   length(trimn(translate(format,' ','.0123456789')))))||".format.html")
                then "<a href=""&rep_root\formats\"||lowcase(substr(format,1,length(trimn(
                    translate(format,' ','.0123456789')))))||'.format.html">'||trim(format)||'</a>'
                else format end
            end
        else '' end
    as format label='Format'
from __profilevar as a left join (select type as fmttype,name,label,format from dictionary.columns
where libname="%ds_libname(&data)" and memname="%ds_memname(&data)") as b
on upcase(a.name)=upcase(b.name) order by type,iv descending;
quit;

ods listing close;
ods html file="&rep_out&dirsep&rep_name..html" (title="Profiles for target %ds_libname(&data).%ds_memname(&data).%upcase(&target)")
    stylesheet=(url="../&rep_css");

proc print data=__profilevar label noobs;
id type;
var type name iv note format;
title "Profiles for target %ds_libname(&data).%ds_memname(&data).%upcase(&target)";
%if "&rep_all" ne "" %then footnote link="&rep_all" 'All reports';
%else footnote;;
run;

ods html close;
ods listing;

goptions reset=all;

proc datasets lib=work nolist mt=(data view);
delete __bins __chart __temp __profilevar;
quit;

%mexit:

%mend model_profilevar;
