/*

------------------------------
Makro: VAR_FORMAT
Nzev: Vrac formt promnn
       Returns variable format
Verze: 2.02
Autor: jan.ruzicka@cze.sas.com
------------------------------

-------------------
POPIS - DESCRIPTION
-------------------

Tato makro funkce vrac formt promnn definovan parametry.
This macro function returns format of specied variable.

----------------
SYNTAXE - SYNTAX
----------------

%var_format
(
    data,    *** specifikace datasetu
                 dataset specification
    var,     *** jmno promnn
                 variable name
    debug=no *** vpis chybovch hlek do LOG (YES/NO)
                 errors to LOG (YES/NO)
)

----------------------------------
HISTORIE ZMN - HISTORY OF CHANGES
----------------------------------

01.11.2003  Verze 1.40
            + zapracovn do jednotnho frameworku SAS maker
              rewriten as part of the framework
15.01.2004  Verze 1.41
            + popis v anglitin
              description in english
05.12.2004  Verze 1.50
            + pro dlouh znakov promnn bez formtu piazuje formt $200.
              for long character varibles without format selects $200.
12.11.2007  Verze 2.02

---------------------------------
PLNY NA VYLEPEN - FUTURE PLANS
---------------------------------

-------------------
PKLADY - EXAMPLES
-------------------

*** 1 - existing variables;

%put Format of DATE in SASHELP.AIR is "%var_format(sashelp.air,date)";
%put Format of AIR in SASHELP.AIR is "%var_format(sashelp.air,air)";
%put Format of NAME in SASHELP.CLASS is "%var_format(sashelp.class,name)";

*** 2 - non-existing variables;

%put Format of AIRRRR in SASHELP.AIR is "%var_format(sashelp.air,airrrr,debug=yes)";

----------------
POZNMKY - NOTES
----------------

Pipomnky a npady k vylepen vtny.
Ideas to improve are welcome.

*/

%macro var_format
(
    data,
    var,
    debug=no
) / des='Returns variable format';

%local dsid varnum varfmt;

%let debug=%qupcase(&debug);

%let dsid=%sysfunc(open(&data));
%if &dsid %then %do;
    %let varnum=%sysfunc(varnum(&dsid,&var));
    %if &varnum %then %do;
        %let varfmt=%qsysfunc(varfmt(&dsid,&varnum));
        %if &varfmt ne %then &varfmt;
        %else %if %sysfunc(vartype(&dsid,&varnum))=C %then $%sysfunc(min(%sysfunc(varlen(&dsid,&varnum)),200)).;
        %else BEST.;
    %end;
    %else %if &debug=YES %then %put ERROR: Variable &var not exist in dataset &data..;
    %let dsid=%sysfunc(close(&dsid));
%end;
%else %if &debug=YES %then %put ERROR: Cannot read dataset &data..;

%mend var_format;
