/*

------------------------------
Makro: VAR_LABEL
Nzev: Vrac label promnn
       Returns variable label
Verze: 2.02
Autor: jan.ruzicka@cze.sas.com
------------------------------

-------------------
POPIS - DESCRIPTION
-------------------

Tato makro funkce vrac label promnn definovan parametry.
This macro function returns label of specied variable.

------
SYNTAX
------

%var_label
(
    data,    *** specifikace datasetu
                 dataset specification
    var,     *** jmno promnn
                 variable name
    debug=no *** vpis chybovch hlek do LOG (YES/NO)
                 errors to LOG (YES/NO)
)

----------------------------------
HISTORIE ZMN - HISTORY OF CHANGES
----------------------------------

07.07.2003  Verze 1.00
01.11.2003  verze 1.40
            + zapracovn do jednotnho frameworku SAS maker
              rewriten as part of the framework
15.01.2004  Verze 1.41
            + popis v anglitin
              description in english
12.11.2007  Verze 2.02

---------------------------------
PLNY NA VYLEPEN - FUTURE PLANS
---------------------------------

-------------------
PKLADY - EXAMPLES
-------------------

*** 1 - existing variables;

%put Label of AIR in SASHELP.AIR is "%var_label(sashelp.air,air)";

*** 2 - non-existing variables;

%put Label of AIRRRR in SASHELP.AIR is "%var_label(sashelp.air,airrrr,debug=yes)";

----------------
POZNMKY - NOTES
----------------

Pipomnky a npady k vylepen vtny.
Ideas to improve are welcome.

*/

%macro var_label
(
    data,
    var,
    debug=no
) / des='Returns variable label';

%local dsid varnum;

%let debug=%qupcase(&debug);

%let dsid=%sysfunc(open(&data));
%if &dsid %then %do;
    %let varnum=%sysfunc(varnum(&dsid,&var));
    %if &varnum %then %qsysfunc(varlabel(&dsid,&varnum));
    %else %if &debug=YES %then %put ERROR: Variable &var not exist in dataset &data..;
    %let dsid=%sysfunc(close(&dsid));
%end;
%else %if &debug=YES %then %put ERROR: Cannot read dataset &data..;

%mend var_label;
