/*

------------------------------
Makro: VAR_LENGTH
Nzev: Vrac dlku promnn
       Returns variable length
Verze: 2.02
Autor: jan.ruzicka@cze.sas.com
------------------------------

-------------------
POPIS - DESCRIPTION
-------------------

Tato makro funkce vrac dlku promnn definovan parametry (u znakov promnn s prefixem $).
This macro function returns length of specied variable (for character variable with $ prefix).

----------------
SYNTAXE - SYNTAX
----------------

%var_length
(
    data,    *** specifikace datasetu
                 dataset specification
    var,     *** jmno promnn
                 variable name
    debug=no *** vpis chybovch hlek do LOG (YES/NO)
                 errors to LOG (YES/NO)
)

----------------------------------
HISTORIE ZMN - HISTORY OF CHANGES
----------------------------------

01.11.2003  Verze 1.40
            + zapracovn do jednotnho frameworku SAS maker
              rewriten as part of the framework
15.01.2004  Verze 1.41
            + popis v anglitin
              description in english
12.11.2007  Verze 2.02

---------------------------------
PLNY NA VYLEPEN - FUTURE PLANS
---------------------------------

-------------------
PKLADY - EXAMPLES
-------------------

*** 1 - existing variables;

%put Length of NAME in SASHELP.CLASS is "%var_length(sashelp.class,name)";

*** 1 - non-existing variables;

%put Length of NAMES SASHELP.CLASS is "%var_length(sashelp.class,names,debug=yes)";

----------------
POZNMKY - NOTES
----------------

Pipomnky a npady k vylepen vtny.
Ideas to improve are welcome.

*/

%macro var_length
(
    data,
    var,
    debug=no
) / des='Returns variable length';

%local dsid varnum;

%let debug=%qupcase(&debug);

%let dsid=%sysfunc(open(&data));
%if &dsid %then %do;
    %let varnum=%sysfunc(varnum(&dsid,&var));
    %if &varnum %then %do;
        %if %sysfunc(vartype(&dsid,&varnum))=C %then $%sysfunc(varlen(&dsid,&varnum));
        %else %sysfunc(varlen(&dsid,&varnum));
    %end;
    %else %if &debug=YES %then %put ERROR: Variable &var not exist in dataset &data..;
    %let dsid=%sysfunc(close(&dsid));
%end;
%else %if &debug=YES %then %put ERROR: Cannot read dataset &data..;

%mend var_length;
